#!/usr/bin/perl
##############################################################################
#
#  ETS -- Wolfenstein: Enemy Territory Statistics
#  (c) <cade@bis.bg> 2003
#
#  Developed for play.evrocom.net server ET community
#  http://play.evrocom.net
#
#  This software is distributed under GPL license, see COPYING for details.
#
#  $Id: ets.pl,v 1.3 2003/07/12 13:26:38 cade Exp $
#
##############################################################################
use strict;

###CONFIG#####################################################################

my $html_header = "<html>
        <title>Wolfenstein: Enemy Territory Statistics</title>
        <style>
        <!--
        BODY  { margin: 0px 0px 0px 0px; color: #fff;
                background-color: #111;
                background-image: url(etback.jpg);
                background-repeat: no-repeat;
                background-position top center;
                background-attachment: fixed; }
        TABLE { font-family: monospace; background-color: #888; }
        TR.hh { border: solid 1px #aaa; background-color: #777; font-weight: bold; }
        TR.r1 { border: solid 1px #aaa; background-color: #444; }
        TR.r2 { border: solid 1px #aaa; background-color: #555; }
        A     { font-family: monospace; color: #0ff; text-decoration: none; }
        -->
        </style>
        <center>
        <p>
        <h1>Wolfenstein: Enemy Territory Statistics</h1>
        <p>
        |
        <a href=players.html>players</a> |
        <a href=clans.html>clans</a> |
        <a href=kdr_p.html>players k/d ratio</a> |
        <a href=kdr_c.html>clans k/d ratio</a> |
        <a href=classide.html>sides/classes</a> |
        <a href=wep.html>weapons</a> |
        <a href=uniforms.html>uniforms/disguise</a> |
        <a href=kpm.html>kills per minute</a> |
        <a href=noise_p.html>player noise</a> |
        <a href=noise_c.html>clan noise</a> |
        <p>
        [TIME]
        <p>
        ";

my $html_footer = "<p>
       Thanks to all players at <a href=http://play.evrocom.net>http://play.evrocom.net</a>!<br>
       <i><b>ETS</b> by (c) <a href=mailto:cade\@bis.bg>&lt;cade\@bis.bg&gt;</a> 2003</i>
       </center>
       </html>";

my $html_motd = "<h2>Welcome to server statistics page for play.evrocom.net!</h2>";

###STATIC#####################################################################

my %CLASS = (
            0 => 'SOLDIER',
            1 => 'MEDIC',
            2 => 'ENGINEER',
            3 => 'FIELD_OPS',
            4 => 'COVERT_OPS',
            );
            
my %SIDE  = (
            0 => 'UNKNOWN',
            1 => 'AXIS',
            2 => 'ALLIES',
            3 => 'SPECTATORS',
            );            

###MAIN#######################################################################

die "usage: $0 log-file html-output-dir\n" unless @ARGV == 2;

my $log_file = shift;
my $html_dir = shift;

my %gun;
my %kil;
my %ded;
my %wep;
my %wep2; # weapon usage by clans
my %wep3; # weapon usage by players
my %nam;
my %use;  # weapon usage
my %kdr;  # kill/death ratio
my %kdr2; # kill/death ratio for clans

my %unif_taken; # uniforms taken
my %unif_lost; # uniforms lost

my %noise_p; # noise in words for players :)
my %noise_c; # noise in words for clans :)

my %class_p_last; # last classes per player
my %class_p; # classes per player

my %side_p_last; # last side per player
my %side_p; # side per player

my %global_class;
my %global_side;

my %classes;
my %sides;

my %client; # clients info

my $href = 0;
my %href;

my $minute; # used for top kills per minute
my %kpm_p_temp;
my %kpm_c_temp;
my %kpm_p;
my %kpm_c;

my $floor = 256;

#-----------------------------------------------------------------------------

print STDERR "processing log file: $log_file\n";

open( my $i, $log_file ) or die "$0: cannot open log file: $log_file\n";
while(<$i>)
  {
  s/^\s*(\d+):(\d+)\s*//; # get time
  my $time = $1 * 60 + $2;
  $minute = $1;
  s/\^.//g; # clear colors etc.
  
  if( /Kill: (\d+) (\d+) (\d+): (.+) killed (.+) by MOD_([A-Z_]+)/i )
    {
    $gun{ $3 } |= $6;
    
    $kpm_p_temp{ $4 }{ $minute }++;
    $kpm_c_temp{ get_clan( $4 ) }{ $minute }++;
    
    die "mismatched gun name: $6" if $gun{ $3 } ne $6;
    $kil{ $4 }++;
    $ded{ $5 }++;
    $wep{ $6 }++;
    $kil{ get_clan( $4 ) }++;
    $ded{ get_clan( $5 ) }++;
    
    # weapon usage
    $wep2{ get_clan( $4 ) }{ $6 }++;
    $wep3{ $4 }{ $6 }++;
    
    # get names...
    $nam{ $4 }++;
    $nam{ $5 }++;
    $nam{ get_clan( $4 ) }++;
    $nam{ get_clan( $5 ) }++;

    print STDERR "unknown: $_\n" if $6 eq '';
    next;
    }
  if( /^say(team)?: ([^:]+): (.+)/i )
    {
    my $p = $2;
    my $cnt = $3 =~ /\S+/g;
    $noise_p{ $p } += $cnt;
    $noise_c{ get_clan( $p ) } += $cnt;
    }
  if( /^ClientUserinfoChanged: (\d+) (.+)/i )
    {
    # n\xwayBG\t\2\c\4\r\0\m\0000000\s\0000000\dn\chromosome\dr\0\w\10\lw\10\sw\41\mu\0\ref\0
    my $id = $1;
    my $data = $2;
    $client{ $id } = { split /\\/, $data };
    
    my $name = $client{ $id }{ 'n' };
    my $dn = $client{ $id }{ 'dn' };
    my $class = $CLASS{ $client{ $id }{ 'c' } };
    my $side = $SIDE{ $client{ $id }{ 't' } };
    if( $dn )
      {
      $unif_taken{ $name }++;
      $unif_lost{ $dn }++;
      };
    if( $class_p_last{ $name } ne $class )
      {
      $class_p{ $name }{ $class }++;
      $class_p_last{ $name } = $class;
      $global_class{ $class }++;
      $classes{ $class }{ $name }++;
      }
    if( $side_p_last{ $name } ne $side )
      {
      $side_p{ $name }{ $side }++;
      $side_p_last{ $name } = $side;
      $global_side{ $side }++;
      $sides{ $side }{ $name }++;
      }
    }
  }
close($i);

print STDERR "done\n";

#-----------------------------------------------------------------------------

print STDERR "processing data...\n";

for( keys %nam )
  {
  my $k = $_;
  $k =~ s/[^!a-z0-9\-\_]/_/gi;
  $href{ $_ } = lc $k;
  }

while( my ( $k, $v ) = each %kpm_p_temp )
  {
  my ( $max_k, $max_v ) = hash_max( $v );
  # $k = sprintf "minute %08d  %s", $max_k, $k;
  $kpm_p{ $k } = $max_v;
  }

while( my ( $k, $v ) = each %kpm_c_temp )
  {
  my ( $max_k, $max_v ) = hash_max( $v );
  # $k = sprintf "minute %08d  %s", $max_k, $k;
  $kpm_c{ $k } = $max_v;
  }

print STDERR "done\n";

#-----------------------------------------------------------------------------

html_save( 'index', $html_motd );

html_save( 'players', mod_player_stats() );
html_save( 'clans', mod_clan_stats() );

for my $k ( keys %classes )
  {
  $href{ $k } = lc $k;
  html_save( 'p_' . $href{$k}, mod_generic_stats( "players as $k", $classes{ $k } ) );
  }
for my $k ( keys %sides )
  {
  $href{ $k } = lc $k;
  html_save( 'p_' . $href{$k}, mod_generic_stats( "players as $k", $sides{ $k } ) );
  }
for my $k ( sort { $kil{ $b } <=> $kil{ $a } } keys %wep2 )
  {
  html_save( 'p_' . $href{$k}, mod_generic_stats( "preferred weapons by clan $k", $wep2{ $k } ) );
  }
for my $k ( sort { $kil{ $b } <=> $kil{ $a } } keys %wep3 )
  {
  html_save( 'p_' . $href{$k}, mod_generic_stats( "sides played by $k", $side_p{ $k } ),
                               mod_generic_stats( "classes played by $k", $class_p{ $k } ),
                               mod_generic_stats( "preferred weapons by player $k", $wep3{ $k } ) );
  }

html_save( 'kdr_p',    mod_generic_stats( "kill/death ratio by players", \%kdr, NOP => 1 ) );
html_save( 'kdr_c',    mod_generic_stats( "kill/death ratio by clans", \%kdr2, NOP => 1 ) );
html_save( 'noise_p',  mod_generic_stats( "noise in words by player", \%noise_p ) );
html_save( 'noise_c',  mod_generic_stats( "noise in words by clan", \%noise_c ) );

html_save( 'uniforms', mod_generic_stats( "uniforms taken", \%unif_taken ),
                       mod_generic_stats( "uniforms lost", \%unif_lost ) );

html_save( 'kpm',      mod_generic_stats( "kills per minute by player", \%kpm_p ),
                       mod_generic_stats( "kills per minute by clan",   \%kpm_c ) );

html_save( 'classide', mod_generic_stats( "sides played", \%global_side ),
                       mod_generic_stats( "classes played", \%global_class ) );

html_save( 'wep', mod_generic_stats( "preferred weapons", \%wep ) );


##############################################################################

sub mod_player_stats
{
  return mod_player_clan_stats(0);
}

sub mod_clan_stats
{
  return mod_player_clan_stats(1);
}

sub mod_player_clan_stats
{
  my $clan = shift;

  my $text;
  
  $text .= "<p>";
  $text .= $clan ? "<h2>Clans</h2>" : "<h2>Players</h2>";
  $text .= "<table width=80% cellspacing=1 cellpadding=5>\n";
  $text .= "<tr class=hh>
               <td width=05% align=right>#</td>
               <td width=15%>Clan</td>
               <td width=30%>Player</td>
               <td width=10% align=right>Kills</td>
               <td width=10% align=right>Deaths</td>
               <td width=10% align=right>K/D</td>
               <td width=20%>Prefered Weapon</td>
            </tr>";
  my $row = 0;
  my $sum_k = 0;
  my $sum_d = 0;
  for my $p ( sort { $kil{ $b } <=> $kil{ $a } } keys %nam )
    {
    next if $clan and $p !~ /^!\[/;
    next if !$clan and $p =~ /^!\[/;

    my $k = $kil{ $p } || 0;
    my $d = $ded{ $p } || 0;
    my $r = $d > 0 ? sprintf "%.2f", $k / $d : 'n/a';
    my $w;
    
    ( $w, undef ) = hash_max( $wep2{ $p } ) if $clan;
    ( $w, undef ) = hash_max( $wep3{ $p } ) if !$clan;
    
    $kdr{ $p } = $r if !$clan;
    $kdr2{ $p } = $r if $clan;
    
    my $c = get_clan( $p );
    
    my $target1 = $href{ $c };
    my $target2 = $href{ $p };
    
    $p =~ s/>/&gt;/g;
    $p =~ s/</&lt;/g;
    
    $sum_k += $k;          
    $sum_d += $d;          
    
    next unless $k > $floor;
    
    $row++;
    my $class = tr_class();
    $text .= "<tr class=$class>
                 <td align=right>$row</td>
                 <td><a href=p_$target1.html>$c</a></td>
                 <td><a href=p_$target2.html>$p</a></td>
                 <td align=right>$k</td>
                 <td align=right>$d</td>
                 <td align=right>$r</td>
                 <td><a href=wep.html>$w</a></td>
              </tr>";
    }
  $text .= "<tr class=hh>
               <td width=05% align=right></td>
               <td width=15%></td>
               <td width=30%></td>
               <td width=10% align=right>$sum_k</td>
               <td width=10% align=right>$sum_d</td>
               <td width=10% align=right></td>
               <td width=20%></td>
            </tr>";
  $text .= "</table>\n";
  $text .= "\n\n\n\n";
  return $text;
}

sub mod_generic_stats
{
  my $title = shift;
  my $hr    = shift;
  my %opt   = @_;
  my $text;
  
  my $sum = hash_sum( $hr );
  
  $text .= "<p>";
  $text .= "<h2>$title</h2>";
  $text .= "<table width=80% cellspacing=1 cellpadding=5>\n";
  $text .= "<tr class=hh>
               <td width=05% align=right>#</td>
               <td width=40%>Name</td>
               <td width=30% align=right>Value</td>
               <td width=25% align=right>Percentage</td>
            </tr>";
  my $row = 0;
  for my $k ( sort { $hr->{ $b } <=> $hr->{ $a } } keys %$hr )
    {
    my $v = $hr->{ $k };
    my $p = $sum > 0 ? sprintf "%.2f%%", 100 * $v / $sum : 'n/a';
    
    if( $href{ $k } )
      {
      $k = "<a href=p_$href{$k}.html>$k</a>";
      }
    
    $p = 'n/a' if $opt{ 'NOP' };
    
    $row++;
    my $class = tr_class();
    $text .= "<tr class=$class>
                 <td align=right>$row</td>
                 <td>$k</td>
                 <td align=right>$v</td>
                 <td align=right>$p</td>
              </tr>";
    }
  $text .= "<tr class=hh>
               <td align=right></td>
               <td>Total:</td>
               <td align=right>$sum</td>
               <td align=right></td>
            </tr>" unless $opt{ 'NOP' };
  $text .= "</table>\n";
  $text .= "\n\n\n\n";
  return $text;
}

##############################################################################

sub html_save
{
  my $file = shift;
  $file = "$html_dir/$file.html";
  open(my $o, '>' . $file) or die "$0: cannot save html file: $file\n";
  my $text = join '', $html_header, @_, $html_footer;
  my $time = scalar localtime();
  $text =~ s/\[TIME\]/$time/g;
  print $o $text;
  close($o);
  print STDERR "file $file written ok\n";
}

my $static_tr;

sub tr_class
{
  $static_tr = !$static_tr;
  $static_tr ? 'r1' : 'r2';
}

sub hash_max
{
  my $hr = shift;
  my $max_k = undef;
  my $max_v = undef;
  while( my ( $k, $v ) = each %$hr )
    {
    if( $max_v eq '' or $v > $max_v )
      {
      $max_k = $k;
      $max_v = $v;
      next;
      }
    }
  #print STDERR "$max_k, $max_v\n";
  return ( $max_k, $max_v );
}

sub hash_sum
{
  my $hr = shift;
  my $sum = 0;
  $sum += $_ for values %$hr;
  return $sum;
}

sub get_clan
{
  my $s = shift;
  my $c = '![]';
  $c = "![$1]" if $s =~ /^\[([^\]]+)\]/;
  $c = "![$1]" if $s =~ /^([A-Z]{2,5})-/i;
  return uc $c;
}

###EOF########################################################################

